print("import: 'glue_vispy_viewers'")
import glue_vispy_viewers

print("import: 'glue_vispy_viewers.scatter'")
import glue_vispy_viewers.scatter

print("import: 'glue_vispy_viewers.volume'")
import glue_vispy_viewers.volume

